﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinToolbars;

namespace WTM_DynamicComboBoxToolWidth
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var comboTool = ultraToolbarsManager1.Tools["ComboBoxTool1"];
            comboTool.SharedProps.MinWidth = 250;
            comboTool.SharedProps.MaxWidth = 500;
            comboTool.SharedProps.Width = 250;
        }

        private void ultraToolbarsManager1_ToolValueChanged(object sender, Infragistics.Win.UltraWinToolbars.ToolEventArgs e)
        {
            if (e.Tool is ComboBoxTool)
            {
                var comboTool = e.Tool as ComboBoxTool;

                using (var bmp = new Bitmap(1, 1))
                {
                    using (var g = Graphics.FromImage(bmp))
                    {
                        if (comboTool.Value != null)
                        {
                            //get the editor (the combo itself in order to check the width
                            var comboType = comboTool.GetType();
                            var pi = comboType.GetProperty("Editor", System.Reflection.BindingFlags.NonPublic 
                                | System.Reflection.BindingFlags.Instance);
                            var editor = pi.GetValue(comboTool) as EditorWithCombo;
                            var textBoxWidth = editor.TextBox.Width;

                            //resove the font based on the bold, italic, fontname and fontsize
                            var font = ResolveFont(comboTool);
                            var text = comboTool.Value.ToString();
                            var textWidth = (int)g.MeasureString(text, font).Width;
                            font.Dispose();
                            var newComboWidth = comboTool.WidthResolved - textBoxWidth + textWidth;

                            //this will change the combo width based on the text inside it
                            //if the value is between the MaxWidth and MinWidth of the combo
                            comboTool.SharedProps.Width = comboTool.SharedProps.MaxWidth > 
                                newComboWidth ? newComboWidth : comboTool.SharedProps.MaxWidth;
                            
                        }
                    }                  
                }

            }
        }

        private static System.Drawing.Font ResolveFont(ComboBoxTool comboTool)
        {
            var appData = new AppearanceData();
            comboTool.ResolveAppearance(ref appData);
            var fontSize = appData.FontData.SizeInPoints == 0
                ? 8.8 : appData.FontData.SizeInPoints;
            var bold = appData.FontData.Bold == DefaultableBoolean.True ? true : false;
            var italic = appData.FontData.Italic == DefaultableBoolean.True ? true : false;
            
            FontStyle fontStyle = new FontStyle();
            if (bold)
            {
                fontStyle = FontStyle.Bold;
            }
            if (italic)
            {
                fontStyle |= FontStyle.Italic;
            }
            var font = new Font(appData.FontData.Name, (float)fontSize, fontStyle);
   
            return font;
        }
    }
}
